var topInset = 27;
var bottomInset = (40);

function equalizerView(width, height, isVisible) {
	var height = height || Constants.EqualizerHeight; // Default to global equalizer height
	var width = width || 500; // Default to 500px wide
	var leftEdgeInset = 24;
	var rightEdgeInset = 14;
	var bsp = ((width - leftEdgeInset - rightEdgeInset) * 0.1) - 25; // equalizer band spacing
	var quaterSpacing = ((height - topInset - bottomInset) * 0.25);
	return {
		nsClass: GXDrawView,
		gxValueKeyPath: "",
		gxIntrinsicHeight: height,
		gxIntrinsicWidth: width,
		gxLayerZPosition: 0,
		axIsIgnored: false,
		axTitle: "Equalizer Parameters",
		gxShowSpeedSeconds:0.2,
		gxHideSpeedSeconds:0.3,
		gxIgnoreNextKeyView: (isVisible == false),
		gxLayout: [
			 "H:[eqPresetLabel]-5-[eqPreset(175)]-(-87.5)-[centerV]",
			 "V:|-13-[eqPreset]",
			 "V:[self]-(<=0)-[centerV]&alignCenterX",
		],
		gxChildViews: {
			"eqContainer" : {
				nsClass: "GXDrawView",
				gxIntrinsicHeight: height,
				gxIntrinsicWidth: width,
				gxLayout: [
					"V:|-50-[self]-0-|",
					`H:[b0]-${bsp}-[b1]-${bsp}-[b2]-${bsp}-[b3]-${bsp}-[b4]-${bsp}-[b5]-${bsp}-[b6]-${bsp}-[b7]-${bsp}-[b8]-${bsp}-[b9]-${rightEdgeInset}-|`,
					`V:|-${topInset}-[s1]`,
					`V:|-${topInset + (quaterSpacing)}-[s2]`,
					`V:|-${topInset + (quaterSpacing * 2)}-[s3]`,
					`V:|-${topInset + (quaterSpacing * 3)}-[s4]`,
					`V:|-${topInset + (quaterSpacing * 4)}-[s5]`,
				],
				gxCanDrawSubviewsIntoLayer: true,
				gxWantsLayer: true,
				gxChildViews:
				{
					"b0": makeEqSlider(0,32),
					"b1": makeEqSlider(1,64),
					"b2": makeEqSlider(2,128),
					"b3": makeEqSlider(3,250),
					"b4": makeEqSlider(4,500),
					"b5": makeEqSlider(5,1000),
					"b6": makeEqSlider(6,2000),
					"b7": makeEqSlider(7,4000),
					"b8": makeEqSlider(8,8000),
					"b9": makeEqSlider(9,16000),
					"s1": { "+": "EqStripe", gxLabelText: "+12dB" },
					"s2": { "+": "EqStripe", gxLabelText: "+6dB" },
					"s3": { "+": "EqStripe", gxLabelText: "0dB", gxBackPainter: { nsClass: GXPainter, gxFillColor: "name:eqStripeMiddle" }},
					"s4": { "+": "EqStripe", gxLabelText: "-6dB" },
					"s5": { "+": "EqStripe", gxLabelText: "-12dB" },
				},
			},
			"centerV" : { nsClass: GXDrawView },
			"eqPreset": {
				"+": "PopupMenuView",
				nsClass: "SoundSource.TCEqualizerPopupMenu",
				sparklineColor: "name:fontColor",
				gxMenuPresentationOffset: 0,
				presetsKey: "Equalizer10Presets",
				gxIntrinsicHeight: 27,
				gxBackPainter: {
					"+": "FlatButtonBack",
					gxEdgeInsets: "1.5, 1.5, 1.5, 1.5",
				},
				gxForePainter: {
					"+": "ControlFore.size13.fontColor.left",
					gxEdgeInsets: "10,0,20,0",
					gxIconAlignment: "{0,0.55}",

				}, 
				gxLabelText: "",
				gxValueKeyPath: "",
				axTitle: "Equalizer Preset",
			},
		}, 
	};
	 
};



function makeEqSlider(band, hz)
{
	var hzTitle = "";
	if (hz == 32)
	{
		hzTitle = "Hz";
	}
	if (hz >= 1000)
	{
		hzTitle = hz == 1000 ? "kHz" : "K";
	}
	var hzNumber = hz < 1000 ? hz : (hz / 1000);
	return {
		"+": "EqSlider",
		gxValueKeyPath: "node.band"+band,
		gxSliderValueKeyPath: "node.uiState.displayedBandValue"+band, // Sliders write their current visible postion to this path. This alows us to animate the readout value, along with the slider
		axTitle: hz + " hertz",
		gxLabelText: hzNumber + hzTitle,
		gxReadoutKeyPath: "node.uiState.displayedBandValue"+band,
	};
}

markup.push({
	EqSlider: {
		"+": "SliderView",
		gxSliderAnimationDuration: 1.3,
		isMainUISlider: true,
		gxAnimatesSliderForModelUpdates: true, // makes sliders animate between preset changes
		gxVertical: true,
		gxLayerZPosition: 2,
		gxDetentPainter: null,
		gxDefaultValue: 0.5,
		gxSliderTrackStyling: 2,
		gxDetentSnapDistance: 0.7,
		gxDetents: "(0.5)",
		axFormatter: { nsClass: "GXScaledFormatter", offset: -0.5, scale: 24, format: "%+ .1f dB", zeroFormat: "0" },
		gxKeystrokeStepSize: 0.00416666667,
		gxTrackOffset: -1,
		gxKnobPainter: {
			"+" : "SliderKnob",
			gxEdgeInsets: "0,0,0,0",
		},
		gxTrackPainter: {
			"+" : "SliderTrack",
			gxCornerRadii: "1.5, 1.5, 1.5, 1.5",
			gxDefaultSize: "4,0",    // default size of track determines track width!
			gxEdgeInsets: "0,6,0,6",     // 'off' track (since not on) is inset on right
			gxFillColor: "name:superDarkMinty",
			gxFrameColor: "name:superDarkMinty",
			"on" : {  },
			"disabled": { },
		},
		gxLayout: [
			"V:[selfLabel(14)]-0-[self]-0-[selfReadout(15)]&alignCenterX",
			"V:|-"+(topInset - 10)+"-[self]-"+(bottomInset - 10)+"-|",
			"H:[self(25)]"
		],
		gxLabelTemplate: "LabelView.size10.right",
		gxReadoutTemplate: {
			"+": "LabelView.size10.fullGreen",
			gxAnimateOnKey: "node.isAnimating",
			gxDefaultValue: "0.0",
			gxFormatter: { nsClass: "GXScaledFormatter", offset: -0.5, scale: 24, format: "% .1f", zeroFormat: "0.0" },
			gxIntrinsicWidth: 36,
			gxClickValue: 0.5,
		}, 
	},
	EqStripe: {
		"+": "PanelDivider",
		gxLayout: ["H:|-14-[selfLabel(35)]-15-[self]-20-|&alignCenterY"], 
		gxLabelTemplate: {
			"+": "LabelView.right.top.fontColor.size9",
			gxIntrinsicHeight: 14
		}
	},
});
