
// Color constants that need to be used in multiple places below

var black =  "rgba:51, 51, 51, 100.0";
var red  =  "rgba:255, 73, 73, 100.0";
var white = "rgba:255.0, 255.0, 255.0, 100.0";
var whiteAlpha80 = "rgba:255.0, 255.0, 255.0, 80.0";
var invisible = "rgba:0.0, 0.0, 0.0, 0.0";
var fullGreen = "rgba:35, 136, 107, 100.0";
var fullGreenAlpha80 = "rgba:35, 136, 107, 80.0";
var lightGray = "rgba: 246, 246, 246, 100.0";
var darkGray =  "rgba: 200, 200, 200, 100.0";
var superDarkMintyLightMode = "rgba:162, 204, 191, 100.0"; // Secondary
var darkMintyLightMode = "rgba:195, 225, 216, 100.0"; // Tertiary
var lightMintyLightMode = "rgba:223, 239, 234, 100.0"; // Quaternary
var superLightMintyLightMode = "rgba:244, 250, 248, 100.0"; // Quinary
var appBackgroundDarkMode = "rgba:36.0, 39.0, 38.0, 100.0";
var superDarkMintyDarkMode = "rgba:36, 39, 38, 100.0"; // Secondary
var darkMintyDarkMode = "rgba:52, 64, 60, 100.0"; // Tertiary
var lightMintyDarkMode = "rgba:42, 49, 47, 100.0"; // Quaternary
var superLightMintyDarkMode = "rgba:40, 45, 43, 100.0"; // Quinary

var elementFrameDarkMode = "rgba:77, 102, 95, 100.0"; // Quinary

var mainPopoverHeader = "rgba:52, 57, 56, 100.0";
var elementFillDarkMode = "rgba:57, 68, 65, 100.0";

function colorComponentsFromString(cString)
{
	var comps = cString.split(":");
	return comps[1].split(",");
}

function blendColorValue(a, b, t)
{
	return Math.sqrt((1 - t) * Math.pow(a,2) + t * Math.pow(b,2));
}

function blendAlphaValue(a, b, t)
{
	return (1-t)*a + t*b;
}

function blendColors(c1, c2, t)
{
	var cc1 = colorComponentsFromString(c1);
	var cc2 = colorComponentsFromString(c2);
	var ret = cc1;
	for (var i = 0; i < cc1.length; i++)
	{
		ret[i] = blendColorValue(cc1[i], cc2[i], t);
	}
	ret[3] = blendAlphaValue(cc1[3], cc1[3], t);
	return "rgba:" + ret.join(",");
}

function generateDesaturatedColors(dict)
{
	for (var key in dict)
	{
		if (key.indexOf(".desaturated") !== -1)
		{
			console.log("skipping");
			continue;
		}
		var lightMode = key.indexOf(".lightMode") !== -1;
		var darkMode = key.indexOf(".darkMode") !== -1;
		
		var newKey = key + ".desaturated";
		if (lightMode)
		{
			dict[newKey] = blendColors(dict[key],white, 0.3);
		}
		else if (darkMode)
		{
			dict[newKey] = blendColors(dict[key],black, 0.3);
		}
		else
		{
			dict[newKey] = blendColors(dict[key],white, 0.3);
		}
		
	}
	return dict;
};

function generatePressedColors(dict)
{
	for (var key in dict)
	{
		if (key.indexOf(".pressed") !== -1)
		{
			console.log("skipping");
			continue;
		}
		var newKey = key + ".pressed";
		dict[newKey] = blendColors(dict[key],invisible, 0.2);
	}
	return dict;
};

function colorWithAlpha(c,a)
{
	var components = colorComponentsFromString(c);
	components[3] = a;
	return "rgba:" + components.join(",");
}

function colorsDict()
{
	var colors = {
		
		"fontColor.lightMode" : black,
		"fontColor.darkMode" : white,
		
		"fontColorInverted.lightMode" : white,
		"fontColorInverted.darkMode" : black,
		
		"background.lightMode": white,
		"background.darkMode": appBackgroundDarkMode,
		
		"black" :  black,
		"red" :  red,
		"white" : white,
		"whiteAlpha80" : whiteAlpha80,
		"invisible" : invisible,
		
		"desat.lightMode" : colorWithAlpha(white, 30.0),
		"desat.darkMode" : colorWithAlpha(appBackgroundDarkMode, 30.0),
		
		"lightGray" : lightGray,
		"darkGray" : darkGray,
		"darkGrayAlpha20" : colorWithAlpha(darkGray, 20.0),
		
		// MARK: Green Shades - Light Mode
		
		"genericPluginPopoverFill.lightMode" :  "rgba: 246, 246, 246, 100.0",
		"genericPluginPopoverFill.darkMode" :  superDarkMintyDarkMode,
		
		"genericPluginPopoverFrame.lightMode" :  white,
		"genericPluginPopoverFrame.darkMode" :  superLightMintyDarkMode,
		
		"genericPluginPopoverFrame.lightMode.inactive" :  white,
		"genericPluginPopoverFrame.darkMode.inactive" :  blendColors(superLightMintyDarkMode,black, 0.8),
		
		"speakerFill.lightMode" : black,
		"speakerFill.darkMode" : white,
		
		"speakerFill.lightMode.muted" : red,
		"speakerFill.darkMode.muted" : red,
		
		"fullGreen" : fullGreen,  // Primary
		"fullGreenAlpha80" : fullGreenAlpha80,  // Primary
		"superDarkMinty.lightMode" : superDarkMintyLightMode, // Secondary
		"darkMinty.lightMode" : darkMintyLightMode, // Tertiary
		"lightMinty.lightMode" : lightMintyLightMode, // Quaternary
		"superLightMinty.lightMode" : superLightMintyLightMode, // Quinary
		
		"superDarkMinty.darkMode" : superDarkMintyDarkMode, // Secondary
		"darkMinty.darkMode" : darkMintyDarkMode, // Tertiary
		"lightMinty.darkMode" : lightMintyDarkMode, // Quaternary
		"superLightMinty.darkMode" : superLightMintyDarkMode, // Quinary
		
		"mainPopoverHeaderFill.lightMode" : fullGreen, // fullGreen
		"mainPopoverHeaderFill.darkMode" : mainPopoverHeader,
		
		"mainPopoverFooterFill.lightMode" : white,
		"mainPopoverFooterFill.darkMode" : mainPopoverHeader,
		
		"trialButtonOrange.lightMode" : "rgba:255,73,73,100",
		"trialButtonOrange.darkMode" : "rgba:255,107,73,100",
		
		"pluginFill.lightMode" : superLightMintyLightMode,
		"pluginFill.darkMode" : lightMintyDarkMode, 
		"pluginFillShowing.lightMode" : darkMintyLightMode,
		"pluginFillShowing.darkMode" : elementFillDarkMode,
		
		"pluginFrame.lightMode" : darkMintyLightMode,
		"pluginFrame.darkMode" : elementFrameDarkMode,
		"pluginFrameSelected.lightMode" : darkMintyLightMode,
		"pluginFrameSelected.darkMode" : elementFrameDarkMode,
		"pluginFrameFocused.lightMode" : superDarkMintyLightMode,
		"pluginFrameFocused.darkMode" : elementFrameDarkMode,
		"pluginFrameShowing.lightMode" : superDarkMintyLightMode,
		"pluginFrameShowing.darkMode" : fullGreen,
		
		// Disabled Look
		
		"pluginFill.lightMode.disabled" : white,
		"pluginFill.darkMode.disabled" : black,
		
		"pluginFrame.lightMode.disabled" : darkGray,
		"pluginFrame.darkMode.disabled" : darkGray,
		
		"eqStripeMiddle.lightMode" : superDarkMintyLightMode,
		"eqStripeMiddle.darkMode" : darkMintyDarkMode,
		
		// Buttons
		"controlBackFill.lightMode" : white,
		"controlBackFill.darkMode" : elementFillDarkMode,
		
		"controlFrame.lightMode" : "rgba:195, 225, 216, 100.0",
		"controlFrame.darkMode" : elementFrameDarkMode,
		
		"btSpinner.lightMode" : black,
		"btSpinner.darkMode" : white,
		
		"menuBarMeterFill.systemLight" : black,
		"menuBarMeterFill.systemDark" : white,
	};
	 
	generatePressedColors(colors); // mutates
	
	// Don't generate pressed colors for the following
	colors = merge(colors, {
		  "volumeFont.lightMode" : black,
		  "volumeFont.darkMode" : white,
		  "volumeFont.lightMode.boost" : red, // Red
		  "volumeFont.darkMode.boost" : red, // Red
	})
	
	return colors;
};



markup.push({
			"Colors" : colorsDict()
});
