let EffectViews = {
	"effectTitle": {
		"+": "LabelView.left.size13",
		gxIntrinsicHeight: 25,
		gxBaselineOffset: 7,
		gxValueKeyPath: "name",
		axIsIgnored: true,
	},
	"effectBypass": {
		nsClass: "SoundSource.TCGXSwitchView",
		axCheckboxRole: true,
		gxValueKeyPath: "enabled",
		gxImageBaseName: "switch",
		axTitle: "Effect Bypass",
		gxToolTipKeyPath: "bypassToolTip"
	},
	"effectRemove": {
		"+" : "deleteButton",
		axTitle: "Effect Delete",
		gxBaselineOffset: 5,
		gxToolTipKeyPath: "removeToolTip"
	},
};

let EffectViewsLayout = [
	"V:|-15-[effectBypass]",
	"H:|-23.5-[effectBypass(42)]-8.5-[effectTitle]-10-[effectRemove(20)]-13-|&alignBaseline",
	`H:|-19-[self(>=${Constants.popoverWidth - 76})]-14-|`,
];

// Markup keys. We use these are automatically referenced in swift


Constants.EqualizerEffectTitle = "10 Band Equalizer";
Constants.VolumeOverdriveEffectTitle = "Volume Overdrive";
Constants.TCVolumeOverdriveMarkupKey = "TCVolumeOverdriveMarkup";
Constants.TC10BandEQMarkupKey = "TC10BandEQMarkup";
Constants.TCAudioUnitModelMarkupKey = "TCAudioUnitModelMarkup";

// The order of these determine how they appear in the "Add Effects" menu
//
Constants.BuiltInEffectMarkup = [ Constants.TCVolumeOverdriveMarkupKey,  Constants.TC10BandEQMarkupKey ];

markup.push({
	
	// MARK: Base Effect Markup
			
	TCEffectModelMarkup : {
		nsClass: "SoundSource.TCModelEffect",
		effectName: "",
		nodeClass: "AHAudioNode",
		effectViewMarkup: "EffectsListItemView",
	},
			 
    "EffectsListItemView" : {
        nsClass: "SoundSource.TCEffectListView",
		effectName: null, // Default to node title
        gxDragController: { "+" : "ListDragController", gxInsertionViewOffset: "{0,-1}", gxCreateDragSpacerView: true },
        drawsSelected: true,
		gxDrawsFocused: true,
		multipleSelection: false,
		axIsIgnored: false,
		gxValueKeyPath: "",
		gxLayout : EffectViewsLayout,
		gxIntrinsicHeight: Constants.effectHeight,
		axTitleKeyPath: "axDescription",
		axHelpKeyPath: "axHelp",
        gxBackPainter: {
            nsClass: "GXRoundRectPainter",
            gxCornerRadius: 6,
            gxFillColor: "name:pluginFill",
			gxFrameColor: "name:pluginFrame",
			"pressed" : {  },
			"selected" : { gxFillColor: "name:pluginFill", gxFrameColor: "name:fullGreen", gxStrokeWidth: 2, },
			"showing" : { gxFillColor: "name:pluginFillShowing", gxFrameColor: "name:fullGreen", gxStrokeWidth: 2 },
			gxStrokeWidth: 1,
            gxEdgeInsets: "4,4,4,6",
        },
		gxChildViews: EffectViews
    },
			
	// MARK: Audio Unit Plugin Markup
			
	[Constants.TCAudioUnitModelMarkupKey] : {
		"+" : "TCEffectModelMarkup",
		nodeClass: "SoundSource.TCAudioUnitNode",
		effectViewMarkup: "PluginListItemView",
	},
			
    "PluginListItemView" : {
        "+" : "EffectsListItemView",
        gxPopoverInfo: "GenericAUPopover",
        axRoleDescription: "Audio Unit effect",
        gxToolTipKeyPath: "popoverToolTip",
        gxDisableOnKey: "node.pluginDisabled",
		gxStylesKeyPath: "customStyles",
    },
		
	// MARK: 10 Band EQ
			
	[Constants.TC10BandEQMarkupKey] : {
		"+" : "TCEffectModelMarkup",
		effectName: Constants.EqualizerEffectTitle,
		nodeClass: "SoundSource.TCEqualizer10Node",
		defaultNodeSettings: {
			"loadedPreset" : "Flat (Off)",
			"enabled" : true
		},
		effectViewMarkup: "10BandEQPluginView",
	},

    "10BandEQPluginView" : {
        "+" : "EffectsListItemView",
		axDescriptionKeyPath: "node.eqDescription",
		axRoleDescription: "\n10-Band Equalizer",
		axHelpKeyPath: "node.eqHelp",
		gxIntrinsicHeight: Constants.EqualizerHeight + 50,
		gxLayout: EffectViewsLayout.concat([
			"H:|-0-[eqView]-0-|",
			"V:|-0-[eqView]-0-|",
		]),
        gxChildViews : merge({
			"eqView" : equalizerView(600, Constants.EqualizerHeight, true),
        }, EffectViews)
    },
			
	// MARK: Volume Overdrive
			
	[Constants.TCVolumeOverdriveMarkupKey] : {
		"+" : "TCEffectModelMarkup",
		effectName: Constants.VolumeOverdriveEffectTitle,
		nodeClass: "SoundSource.TCGainEffectsNode",
		defaultNodeSettings: {
			"gain" : 1.0,
			"overdrive" : 1.0,
			"useVolumeCurve" : true,
			"limitPeaks" : true,
		},
		effectViewMarkup : "OverdrivePluginView"
	},
			
	"OverdrivePluginView" : {
		"+" : "EffectsListItemView",
		gxLayout: EffectViewsLayout.concat([
			"H:[effectBypass]-(>=0)-[off(35)]-0-[x2]-0-[x3]-0-[x4]&alignCenterY",
			"V:[center]-(<=0)-[self]&alignCenterX",
			"H:[off]-35-[center]",
		    "V:[off(20)]",
			"V:[x2(==off)]",
			"V:[x3(==off)]",
			"V:[x4(==off)]",
			"H:[x2(==off)]",
			"H:[x3(==off)]",
			"H:[x4(==off)]",
		]),
		gxChildViews : merge({
			"center" : { nsClass: "GXDrawView", },
			label: {
				 "+": "LabelView.mid.size105",
				 gxFrameRect: "18,66,83,23",
				 gxDefaultValue: "Overdrive"
			},
			off: {	"+": "OverDriveButton.leftSegment", gxTitle: "1\u{00D7}", axTitle: "overdrive 1X",
						 gxValueKeyPath: "node.overdrive_x_1" },
			x2: {	"+": "OverDriveButton.middleSegment", gxTitle: "2\u{00D7}", axTitle: "overdrive 2X",
						 gxValueKeyPath: "node.overdrive_x_2" },
			x3: {	"+": "OverDriveButton.middleSegment", gxTitle: "3\u{00D7}", axTitle: "overdrive 3X",
						 gxValueKeyPath: "node.overdrive_x_3" },
			x4: {	"+": "OverDriveButton.rightSegment", gxTitle: "4\u{00D7}", axTitle: "overdrive 4X",
						 gxValueKeyPath: "node.overdrive_x_4" }
		}, EffectViews)
	},

	"OverDriveButton" : {
		"+" : "ButtonView",
		gxBackPainter: {
			"+" : "ControlBack",
			gxFrameColor: "name:pluginFrameFocused",
			gxFillColor: "name:superLightMinty",
			"on" : {
				gxFillColor: "name:fullGreen"
			},
		},
		gxForePainter: {
			"+" : "ControlFore",
			"darkMode" : {
				gxFontColor: "name:white",
			},
			"lightMode" : {
				gxFontColor: "name:black",
			},
			"on" : {
				gxFontColor: "name:white",
				gxFontWeight: 0.4,
			},
		}
	}
})

